@@ -8,7 +8,7 @@
 
 #include <asm/types.h>		/* For __uXX types */
 
-#define IP_VS_VERSION_CODE	0x010101
+#define IP_VS_VERSION_CODE	0x010102
 #define NVERSION(version)			\
 	(version >> 16) & 0xFF,			\
 	(version >> 8) & 0xFF,			\
@@ -888,14 +888,14 @@ extern int ip_vs_leave(struct ip_vs_service *svc, struct sk_buff *skb,
  */
 extern int sysctl_ip_vs_cache_bypass;
 extern int sysctl_ip_vs_expire_nodest_conn;
-extern int sysctl_ip_vs_sync_threshold;
+extern int sysctl_ip_vs_sync_threshold[2];
 extern int sysctl_ip_vs_nat_icmp_send;
 extern atomic_t ip_vs_dropentry;
 extern struct ip_vs_stats ip_vs_stats;
 
-extern struct ip_vs_service *ip_vs_service_get(__u32 fwmark,
-					       __u16 protocol,
-					       __u32 vaddr, __u16 vport);
+extern struct ip_vs_service *
+ip_vs_service_get(__u32 fwmark, __u16 protocol, __u32 vaddr, __u16 vport);
+
 static inline void ip_vs_service_put(struct ip_vs_service *svc)
 {
 	atomic_dec(&svc->usecnt);
@@ -904,7 +904,6 @@ static inline void ip_vs_service_put(struct ip_vs_service *svc)
 extern struct ip_vs_dest *
 ip_vs_lookup_real_service(__u16 protocol, __u32 daddr, __u16 dport);
 extern void update_defense_level(void);
-extern void ip_vs_random_dropentry(void);
 extern int ip_vs_control_init(void);
 extern void ip_vs_control_cleanup(void);
 
