@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.29 2002/09/15 08:14:08 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.30 2002/11/30 01:50:35 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -498,7 +498,7 @@ static void ip_vs_conn_expire(unsigned long data)
 	 */
 	if (likely(atomic_read(&cp->refcnt) == 1)) {
 		/* make sure that there is no timer on it now */
-		if (timer_pending(&cp->timer))
+		if (sltimer_pending(&cp->timer))
 			del_sltimer(&cp->timer);
 
 		/* does anybody control me? */
@@ -556,7 +556,7 @@ ip_vs_conn_new(int proto, __u32 caddr, __u16 cport, __u32 vaddr, __u16 vport,
 
 	memset(cp, 0, sizeof(*cp));
 	INIT_LIST_HEAD(&cp->c_list);
-	init_timer(&cp->timer);
+	init_sltimer(&cp->timer);
 	cp->timer.data     = (unsigned long)cp;
 	cp->timer.function = ip_vs_conn_expire;
 	cp->protocol	   = proto;
