@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_core.c,v 1.32 2002/09/15 08:14:08 wensong Exp $
+ * Version:     $Id: ip_vs_core.c,v 1.33 2002/12/19 08:40:52 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -1000,7 +1000,8 @@ static unsigned int ip_vs_in(unsigned int hooknum,
 	if (ip_vs_sync_state == IP_VS_STATE_MASTER &&
 	    (cp->protocol != IPPROTO_TCP ||
 	     cp->state == IP_VS_TCP_S_ESTABLISHED) &&
-	    (atomic_read(&cp->in_pkts) % 50 == sysctl_ip_vs_sync_threshold))
+	    (atomic_read(&cp->in_pkts) % sysctl_ip_vs_sync_threshold[1]
+	     == sysctl_ip_vs_sync_threshold[0]))
 		ip_vs_sync_conn(cp);
 
 	ip_vs_conn_put(cp);
