@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.32 2002/11/30 01:50:35 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.33 2002/12/19 08:40:52 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -78,9 +78,10 @@ static int sysctl_ip_vs_amemthresh = 1024;
 static int sysctl_ip_vs_am_droprate = 10;
 int sysctl_ip_vs_cache_bypass = 0;
 int sysctl_ip_vs_expire_nodest_conn = 0;
-int sysctl_ip_vs_sync_threshold = 3;
+int sysctl_ip_vs_sync_threshold[2] = { 3, 50 };
 int sysctl_ip_vs_nat_icmp_send = 0;
 
+
 #ifdef CONFIG_IP_VS_DEBUG
 static int sysctl_ip_vs_debug_level = 0;
 
@@ -1327,14 +1328,15 @@ static int ip_vs_zero_all(void)
 }
 
 
-static int ip_vs_sysctl_defense_mode(ctl_table *ctl, int write,
-	struct file * filp, void *buffer, size_t *lenp)
+static int
+proc_do_defense_mode(ctl_table *table, int write, struct file * filp,
+		     void *buffer, size_t *lenp)
 {
-	int *valp = ctl->data;
+	int *valp = table->data;
 	int val = *valp;
-	int ret;
+	int rc;
 
-	ret = proc_dointvec(ctl, write, filp, buffer, lenp);
+	rc = proc_dointvec(table, write, filp, buffer, lenp);
 	if (write && (*valp != val)) {
 		if ((*valp < 0) || (*valp > 3)) {
 			/* Restore the correct value */
@@ -1345,7 +1347,27 @@ static int ip_vs_sysctl_defense_mode(ctl_table *ctl, int write,
 			local_bh_enable();
 		}
 	}
-	return ret;
+	return rc;
+}
+
+
+static int
+proc_do_sync_threshold(ctl_table *table, int write, struct file *filp,
+		       void *buffer, size_t *lenp)
+{
+	int *valp = table->data;
+	int val[2];
+	int rc;
+
+	/* backup the value first */
+	memcpy(val, valp, sizeof(val));
+
+	rc = proc_dointvec(table, write, filp, buffer, lenp);
+	if (write && (valp[0] < 0 || valp[1] < 0 || valp[0] >= valp[1])) {
+		/* Restore the correct value */
+		memcpy(valp, val, sizeof(val));
+	}
+	return rc;
 }
 
 
@@ -1375,13 +1397,13 @@ static struct ip_vs_sysctl_table ipv4_vs_table = {
 	  &proc_dointvec},
 	 {NET_IPV4_VS_DROP_ENTRY, "drop_entry",
 	  &sysctl_ip_vs_drop_entry, sizeof(int), 0644, NULL,
-	  &ip_vs_sysctl_defense_mode},
+	  &proc_do_defense_mode},
 	 {NET_IPV4_VS_DROP_PACKET, "drop_packet",
 	  &sysctl_ip_vs_drop_packet, sizeof(int), 0644, NULL,
-	  &ip_vs_sysctl_defense_mode},
+	  &proc_do_defense_mode},
 	 {NET_IPV4_VS_SECURE_TCP, "secure_tcp",
 	  &sysctl_ip_vs_secure_tcp, sizeof(int), 0644, NULL,
-	  &ip_vs_sysctl_defense_mode},
+	  &proc_do_defense_mode},
 #if 0
 	 {NET_IPV4_VS_TO_ES, "timeout_established",
 	  &vs_timeout_table_dos.timeout[IP_VS_S_ESTABLISHED],
@@ -1427,8 +1449,8 @@ static struct ip_vs_sysctl_table ipv4_vs_table = {
 	  &sysctl_ip_vs_expire_nodest_conn, sizeof(int), 0644, NULL,
 	  &proc_dointvec},
 	 {NET_IPV4_VS_SYNC_THRESHOLD, "sync_threshold",
-	  &sysctl_ip_vs_sync_threshold, sizeof(int), 0644, NULL,
-	  &proc_dointvec},
+	  &sysctl_ip_vs_sync_threshold, sizeof(sysctl_ip_vs_sync_threshold),
+	  0644, NULL, &proc_do_sync_threshold},
 	 {NET_IPV4_VS_NAT_ICMP_SEND, "nat_icmp_send",
 	  &sysctl_ip_vs_nat_icmp_send, sizeof(int), 0644, NULL,
 	  &proc_dointvec},
