@@ -1,7 +1,7 @@
 /*
  * IPVS:        Locality-Based Least-Connection with Replication scheduler
  *
- * Version:     $Id: ip_vs_lblcr.c,v 1.10 2002/03/25 12:44:35 wensong Exp $
+ * Version:     $Id: ip_vs_lblcr.c,v 1.11 2002/09/15 08:14:08 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@gnuchina.org>
  *
@@ -183,6 +183,9 @@ static inline struct ip_vs_dest *ip_vs_dest_set_min(struct ip_vs_dest_set *set)
 	/* select the first destination server, whose weight > 0 */
 	for (e=set->list; e!=NULL; e=e->next) {
 		least = e->dest;
+		if (least->flags & IP_VS_DEST_F_OVERLOAD)
+			continue;
+
 		if ((atomic_read(&least->weight) > 0)
 		    && (least->flags & IP_VS_DEST_F_AVAILABLE)) {
 			loh = atomic_read(&least->activeconns) * 50
@@ -197,6 +200,9 @@ static inline struct ip_vs_dest *ip_vs_dest_set_min(struct ip_vs_dest_set *set)
   nextstage:
 	for (e=e->next; e!=NULL; e=e->next) {
 		dest = e->dest;
+		if (dest->flags & IP_VS_DEST_F_OVERLOAD)
+			continue;
+
 		doh = atomic_read(&dest->activeconns) * 50
 			+ atomic_read(&dest->inactconns);
 		if ((loh * atomic_read(&dest->weight) >
@@ -728,6 +734,9 @@ __ip_vs_wlc_schedule(struct ip_vs_service *svc, struct iphdr *iph)
 	l = &svc->destinations;
 	for (e=l->next; e!=l; e=e->next) {
 		least = list_entry(e, struct ip_vs_dest, n_list);
+		if (least->flags & IP_VS_DEST_F_OVERLOAD)
+			continue;
+
 		if (atomic_read(&least->weight) > 0) {
 			loh = atomic_read(&least->activeconns) * 50
 				+ atomic_read(&least->inactconns);
@@ -742,6 +751,9 @@ __ip_vs_wlc_schedule(struct ip_vs_service *svc, struct iphdr *iph)
   nextstage:
 	for (e=e->next; e!=l; e=e->next) {
 		dest = list_entry(e, struct ip_vs_dest, n_list);
+		if (dest->flags & IP_VS_DEST_F_OVERLOAD)
+			continue;
+
 		doh = atomic_read(&dest->activeconns) * 50
 			+ atomic_read(&dest->inactconns);
 		if (loh * atomic_read(&dest->weight) >
@@ -847,14 +859,13 @@ ip_vs_lblcr_schedule(struct ip_vs_service *svc, struct iphdr *iph)
  */
 static struct ip_vs_scheduler ip_vs_lblcr_scheduler =
 {
-	{0},                     /* n_list */
-	"lblcr",                 /* name */
-	ATOMIC_INIT(0),          /* refcnt */
-	THIS_MODULE,             /* this module */
-	ip_vs_lblcr_init_svc,    /* service initializer */
-	ip_vs_lblcr_done_svc,    /* service done */
-	ip_vs_lblcr_update_svc,  /* service updater */
-	ip_vs_lblcr_schedule,    /* select a server from the destination list */
+	.name =			"lblcr",
+	.refcnt =		ATOMIC_INIT(0),
+	.module =		THIS_MODULE,
+	.init_service =		ip_vs_lblcr_init_svc,
+	.done_service =		ip_vs_lblcr_done_svc,
+	.update_service =	ip_vs_lblcr_update_svc,
+	.schedule =		ip_vs_lblcr_schedule,
 };
 
 
