@@ -1,7 +1,7 @@
 /*
  * IPVS:        Least-Connection Scheduling module
  *
- * Version:     $Id: ip_vs_lc.c,v 1.8 2001/10/19 15:05:17 wensong Exp $
+ * Version:     $Id: ip_vs_lc.c,v 1.9 2002/09/15 08:14:08 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -69,6 +69,8 @@ ip_vs_lc_schedule(struct ip_vs_service *svc, struct iphdr *iph)
 	l = &svc->destinations;
 	for (e=l->next; e!=l; e=e->next) {
 		least = list_entry (e, struct ip_vs_dest, n_list);
+		if (least->flags & IP_VS_DEST_F_OVERLOAD)
+			continue;
 		if (atomic_read(&least->weight) > 0) {
 			lac = (atomic_read(&least->activeconns) << 5)
 				+ atomic_read(&least->inactconns);
@@ -83,7 +85,8 @@ ip_vs_lc_schedule(struct ip_vs_service *svc, struct iphdr *iph)
   nextstage:
 	for (e=e->next; e!=l; e=e->next) {
 		dest = list_entry(e, struct ip_vs_dest, n_list);
-		if (atomic_read(&dest->weight) == 0)
+		if ((dest->flags & IP_VS_DEST_F_OVERLOAD) ||
+		    atomic_read(&dest->weight) == 0)
 			continue;
 		dac = (atomic_read(&dest->activeconns) << 5)
 			+ atomic_read(&dest->inactconns);
@@ -103,14 +106,13 @@ ip_vs_lc_schedule(struct ip_vs_service *svc, struct iphdr *iph)
 
 
 static struct ip_vs_scheduler ip_vs_lc_scheduler = {
-	{0},			/* n_list */
-	"lc",			/* name */
-	ATOMIC_INIT(0),	/* refcnt */
-	THIS_MODULE,		/* this module */
-	ip_vs_lc_init_svc,	/* service initializer */
-	ip_vs_lc_done_svc,	/* service done */
-	ip_vs_lc_update_svc,	/* service updater */
-	ip_vs_lc_schedule,	/* select a server from the destination list */
+	.name =			"lc",
+	.refcnt =		ATOMIC_INIT(0),
+	.module =		THIS_MODULE,
+	.init_service =		ip_vs_lc_init_svc,
+	.done_service =		ip_vs_lc_done_svc,
+	.update_service =	ip_vs_lc_update_svc,
+	.schedule =		ip_vs_lc_schedule,
 };
 
 
