@@ -1,7 +1,7 @@
 /*
  * IPVS:        Round-Robin Scheduling module
  *
- * Version:     $Id: ip_vs_rr.c,v 1.7 2001/06/04 13:51:03 wensong Exp $
+ * Version:     $Id: ip_vs_rr.c,v 1.9 2002/09/15 08:14:08 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -68,12 +68,15 @@ ip_vs_rr_schedule(struct ip_vs_service *svc, struct iphdr *iph)
 	p = p->next;
 	q = p;
 	do {
+		/* skip list head */
 		if (q == &svc->destinations) {
 			q = q->next;
 			continue;
 		}
+		
 		dest = list_entry(q, struct ip_vs_dest, n_list);
-		if (atomic_read(&dest->weight) > 0)
+		if (!(dest->flags & IP_VS_DEST_F_OVERLOAD) &&
+		    atomic_read(&dest->weight) > 0)
 			/* HIT */
 			goto out;
 		q = q->next;
@@ -95,14 +98,13 @@ ip_vs_rr_schedule(struct ip_vs_service *svc, struct iphdr *iph)
 
 
 static struct ip_vs_scheduler ip_vs_rr_scheduler = {
-	{0},			/* n_list */
-	"rr",			/* name */
-	ATOMIC_INIT(0),		/* refcnt */
-	THIS_MODULE,		/* this module */
-	ip_vs_rr_init_svc,	/* service initializer */
-	ip_vs_rr_done_svc,	/* service done */
-	ip_vs_rr_update_svc,	/* service updater */
-	ip_vs_rr_schedule,	/* select a server from the destination list */
+	.name =			"rr",			/* name */
+	.refcnt =		ATOMIC_INIT(0),
+	.module =		THIS_MODULE,
+	.init_service =		ip_vs_rr_init_svc,
+	.done_service =		ip_vs_rr_done_svc,
+	.update_service =	ip_vs_rr_update_svc,
+	.schedule =		ip_vs_rr_schedule,
 };
 
 static int __init ip_vs_rr_init(void)
@@ -118,3 +120,4 @@ static void __exit ip_vs_rr_cleanup(void)
 
 module_init(ip_vs_rr_init);
 module_exit(ip_vs_rr_cleanup);
+MODULE_LICENSE("GPL");
