@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_sync.c,v 1.10 2002/11/30 01:50:35 wensong Exp $
+ * Version:     $Id: ip_vs_sync.c,v 1.11 2002/12/19 08:40:52 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -93,7 +93,9 @@ struct ip_vs_sync_conn_options {
       |                    IPVS Sync Connection (n)                   |
       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */
-#define SYNC_MESG_MAX_SIZE      (24*50+4)
+
+#define SYNC_MESG_HEADER_LEN	4
+
 struct ip_vs_sync_mesg {
 	__u8                    nr_conns;
 	__u8                    reserved;
@@ -102,6 +104,8 @@ struct ip_vs_sync_mesg {
 	/* ip_vs_sync_conn entries start here */
 };
 
+/* the maximum length of synchronization message */
+static int sync_mesg_maxlen;
 
 struct ip_vs_sync_buff {
 	struct list_head        list;
@@ -154,14 +158,14 @@ static inline struct ip_vs_sync_buff * ip_vs_sync_buff_create(void)
 	if (!(sb=kmalloc(sizeof(struct ip_vs_sync_buff), GFP_ATOMIC)))
 		return NULL;
 
-	if (!(sb->mesg=kmalloc(SYNC_MESG_MAX_SIZE, GFP_ATOMIC))) {
+	if (!(sb->mesg=kmalloc(sync_mesg_maxlen, GFP_ATOMIC))) {
 		kfree(sb);
 		return NULL;
 	}
 	sb->mesg->nr_conns = 0;
 	sb->mesg->size = 4;
 	sb->head = (unsigned char *)sb->mesg + 4;
-	sb->end = (unsigned char *)sb->mesg + SYNC_MESG_MAX_SIZE;
+	sb->end = (unsigned char *)sb->mesg + sync_mesg_maxlen;
 	sb->firstuse = jiffies;
 	return sb;
 }
@@ -320,7 +324,6 @@ char ip_vs_mcast_ifn[IP_VS_IFNAME_MAXLEN];
 /* multicast addr */
 static struct sockaddr_in mcast_addr;
 
-
 /*
  *      Setup loopback of outgoing multicasts on a sending socket
  */
@@ -369,6 +372,29 @@ static int set_mcast_if(struct sock *sk, char *ifname)
 	return 0;
 }
 
+
+/*
+ *	Set the maximum length of sync message according to the
+ *	specified interface's MTU.
+ */
+static int set_sync_mesg_maxlen(char *ifname)
+{
+	struct net_device *dev;
+	int num;
+
+	if ((dev = __dev_get_by_name(ifname)) == NULL)
+		return -ENODEV;
+
+	num = (dev->mtu - sizeof(struct iphdr) - sizeof(struct udphdr) -
+	       SYNC_MESG_HEADER_LEN - 20) / SIMPLE_CONN_SIZE;
+	sync_mesg_maxlen = SYNC_MESG_HEADER_LEN + SIMPLE_CONN_SIZE * num;
+	IP_VS_DBG(7, "setting the maximum length of sync message %d.\n",
+		  sync_mesg_maxlen);
+
+	return 0;
+}
+
+
 /*
  *      Join a multicast group.
  *      the group is specified by a class D multicast address 224.0.0.0/8
@@ -631,7 +657,7 @@ static void sync_backup_loop(void)
 	char *buf;
 	int len;
 
-	if (!(buf=kmalloc(SYNC_MESG_MAX_SIZE, GFP_ATOMIC))) {
+	if (!(buf=kmalloc(sync_mesg_maxlen, GFP_ATOMIC))) {
 		IP_VS_ERR("sync_backup_loop: kmalloc error\n");
 		return;
 	}
@@ -645,7 +671,7 @@ static void sync_backup_loop(void)
 		/* do you have data now? */
 		while (!skb_queue_empty(&(sock->sk->receive_queue))) {
 			if ((len=ip_vs_receive(sock, buf,
-					       SYNC_MESG_MAX_SIZE))<=0) {
+					       sync_mesg_maxlen))<=0) {
 				IP_VS_ERR("receiving message error\n");
 				break;
 			}
@@ -695,6 +721,9 @@ static int sync_thread(void *startup)
 	recalc_sigpending();
 	spin_unlock_irq(&current->sig->siglock);
 
+	/* set the maximum length of sync message */
+	set_sync_mesg_maxlen(ip_vs_mcast_ifn);
+
 	/* set up multicast address */
 	mcast_addr.sin_family = AF_INET;
 	mcast_addr.sin_port = htons(IP_VS_SYNC_PORT);
