@@ -2,7 +2,7 @@
  *      ipvsadm - IP Virtual Server ADMinistration program
  *                for IPVS NetFilter Module in kernel 2.4
  *
- *      Version: $Id: ipvsadm.c,v 1.20 2002/09/15 08:14:13 wensong Exp $
+ *      Version: $Id: ipvsadm.c,v 1.21 2002/11/30 01:50:37 wensong Exp $
  *
  *      Authors: Wensong Zhang <wensong@linuxvirtualserver.org>
  *               Peter Kese <peter.kese@ijs.si>
@@ -119,7 +119,7 @@
 #include "libipvs/libipvs.h"
 
 #define IPVSADM_VERSION_NO	"v" VERSION
-#define IPVSADM_VERSION_DATE	"2002/09/14"
+#define IPVSADM_VERSION_DATE	"2002/11/15"
 #define IPVSADM_VERSION		IPVSADM_VERSION_NO " " IPVSADM_VERSION_DATE
 
 #define MAX_TIMEOUT		(86400*31)	/* 31 days */
@@ -177,7 +177,8 @@ static const char* cmdnames[] = {
 #define OPT_STATS		0x04000
 #define OPT_RATE		0x08000
 #define OPT_THRESHOLDS		0x10000
-#define NUMBER_OF_OPT		17
+#define OPT_SORT		0x20000
+#define NUMBER_OF_OPT		18
 
 static const char* optnames[] = {
 	"numeric",
@@ -197,6 +198,7 @@ static const char* optnames[] = {
 	"stats",
 	"rate",
 	"thresholds",
+	"sort",
 };
 
 /*
@@ -209,21 +211,21 @@ static const char* optnames[] = {
  */
 static const char commands_v_options[NUMBER_OF_CMD][NUMBER_OF_OPT] =
 {
-	/*   -n   -c   svc  -s   -p   -M   -r   fwd  -w   -x   -y   -mc  tot  dmn  -st  -rt  thr */
-/*ADD*/     {'x', 'x', '+', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*EDIT*/    {'x', 'x', '+', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*DEL*/     {'x', 'x', '+', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*FLUSH*/   {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*LIST*/    {' ', '1', '1', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', '1', '1', ' ', ' ', ' '},
-/*ADDSRV*/  {'x', 'x', '+', 'x', 'x', 'x', '+', ' ', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*DELSRV*/  {'x', 'x', '+', 'x', 'x', 'x', '+', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*EDITSRV*/ {'x', 'x', '+', 'x', 'x', 'x', '+', ' ', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*TIMEOUT*/ {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*STARTD*/  {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', ' ', 'x', 'x', 'x', 'x', 'x'},
-/*STOPD*/   {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*RESTORE*/ {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*SAVE*/    {' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
-/*ZERO*/    {'x', 'x', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+	/*   -n   -c   svc  -s   -p   -M   -r   fwd  -w   -x   -y   -mc  tot  dmn  -st  -rt  thr  srt */
+/*ADD*/     {'x', 'x', '+', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*EDIT*/    {'x', 'x', '+', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*DEL*/     {'x', 'x', '+', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*FLUSH*/   {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*LIST*/    {' ', '1', '1', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', '1', '1', ' ', ' ', ' ', ' '},
+/*ADDSRV*/  {'x', 'x', '+', 'x', 'x', 'x', '+', ' ', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*DELSRV*/  {'x', 'x', '+', 'x', 'x', 'x', '+', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*EDITSRV*/ {'x', 'x', '+', 'x', 'x', 'x', '+', ' ', ' ', ' ', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*TIMEOUT*/ {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*STARTD*/  {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', ' ', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*STOPD*/   {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*RESTORE*/ {'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*SAVE*/    {' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
+/*ZERO*/    {'x', 'x', ' ', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x', 'x'},
 };
 
 /* printing format flags */
@@ -233,6 +235,7 @@ static const char commands_v_options[NUMBER_OF_CMD][NUMBER_OF_OPT] =
 #define FMT_STATS	0x0004
 #define FMT_RATE	0x0008
 #define FMT_THRESHOLDS	0x0010
+#define FMT_SORT	0x0020
 
 #define SERVICE_NONE    0x0
 #define SERVICE_ADDR    0x1
@@ -267,7 +270,7 @@ static char * port_to_anyname(unsigned short port, unsigned short proto);
 static char * addrport_to_anyname(struct in_addr *addr, unsigned short port,
 				  unsigned short proto, unsigned int format);
 static int parse_service(char *buf, u_int16_t proto,
-		  u_int32_t *addr, u_int16_t *port);
+			 u_int32_t *addr, u_int16_t *port);
 static int parse_netmask(char *buf, u_int32_t *addr);
 static int parse_timeout(char *buf, int min, int max);
 static unsigned int parse_fwmark(char *buf);
@@ -375,6 +378,7 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 		{"stats", '\0', POPT_ARG_NONE, NULL, '7'},
 		{"rate", '\0', POPT_ARG_NONE, NULL, '8'},
 		{"thresholds", '\0', POPT_ARG_NONE, NULL, '9'},
+		{"sort", '\0', POPT_ARG_NONE, NULL, '0'},
 		{NULL, 0, 0, NULL, 0}
 	};
 
@@ -558,6 +562,10 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 			set_option(options, OPT_THRESHOLDS);
 			*format |= FMT_THRESHOLDS;
 			break;
+		case '0':
+			set_option(options, OPT_SORT);
+			*format |= FMT_SORT;
+			break;
 		default:
 			fail(2, "invalid option `%s'",
 			     poptBadOption(context, POPT_BADOPTION_NOALIAS));
@@ -643,6 +651,7 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 		{"stats", 0, 0, '7'},
 		{"rate", 0, 0, '8'},
 		{"thresholds", 0, 0, '9'},
+		{"sort", 0, 0, '0'},
 		{"help", 0, 0, 'h'},
 		{0, 0, 0, 0}
 	};
@@ -848,6 +857,10 @@ parse_options(int argc, char **argv, struct ipvs_command_entry *ce,
 			set_option(options, OPT_THRESHOLDS);
 			*format |= FMT_THRESHOLDS;
 			break;
+		case '0':
+			set_option(options, OPT_SORT);
+			*format |= FMT_SORT;
+			break;
 		default:
 			fail(2, "invalid option `%s'", argv[optind-1]);
 		}
@@ -990,6 +1003,10 @@ static int process_options(int argc, char **argv, int reading_stdin)
 		result = ipvs_del_dest(&ce.svc, &ce.dest);
 		break;
 
+	case CMD_TIMEOUT:
+		result = ipvs_set_timeout(&ce.timeout);
+		break;
+
 	case CMD_STARTDAEMON:
 		result = ipvs_start_daemon(&ce.daemon);
 		break;
@@ -1266,6 +1283,7 @@ static void usage_exit(const char *program, const int exit_status)
 		"  --stats                             output of statistics information\n"
 		"  --rate                              output of rate information\n"
 		"  --thresholds                        output of thresholds information\n"
+		"  --sort                              sorting output of service/server entries\n"
 		"  --numeric      -n                   numeric output of addresses and ports\n",
 		DEF_SCHED);
 
@@ -1311,7 +1329,7 @@ static void fail(int err, char *msg, ...)
 
 static int modprobe_ipvs(void)
 {
-	char *argv[] = { "/sbin/modprobe", "-s", "-k", "--", "ip_vs", NULL };
+	char *argv[] = { "/sbin/modprobe", "--", "ip_vs", NULL };
 	int child;
 	int status;
 	int rc;
@@ -1590,7 +1608,10 @@ print_service_entry(ipvs_service_entry_t *se, unsigned int format)
 	printf("\n");
 
 	/* print all the destination entries */
-	for (i=0; i<d->num_dests; i++) {
+	if (format & FMT_SORT)
+		ipvs_sort_dests(d, ipvs_cmp_dests);
+
+	for (i = 0; i < d->num_dests; i++) {
 		struct in_addr daddr;
 		char *dname;
 		ipvs_dest_entry_t *e = &d->entrytable[i];
@@ -1665,8 +1686,11 @@ static void list_all(unsigned int format)
 		exit(1);
 	}
 
+	if (format & FMT_SORT)
+		ipvs_sort_services(get, ipvs_cmp_services);
+
 	print_title(format);
-	for (i=0; i<get->num_services; i++)
+	for (i = 0; i < get->num_services; i++)
 		print_service_entry(&get->entrytable[i], format);
 	free(get);
 }
